%magnetcalc.mo
%Program for calculations of magnetic force in permanent magnets
%based on
%http://www.magnetsales.com/Design/DesignG_frames/frame_dgbod2.htm#stabilit
%y and http://www.magneticsolutions.com.au/cgi-bin/flux-graphs.pl
%Hvard Braa 
%25.02.2011
%Estimates approx up to 30% errors

clear all
clc



magnet_type=4;      %1=cylindrical
                    %2=rectangular
                    %3=ring-shaped
                    %4=two cylindrical attracting eachother
                
force_plot=1;       %1=newton 
                    %2=kg 
                
steel_back_plate=0  %1=yes
                    %0=no

number_of_magnets=1 %quantity of magnet connections

Br=12.3;  %material kiloGauss

N=100; %resolution graph
mm_to_inch=1/25.4;
pounds_to_kg=0.45359;
g=9.81;
distance_axis=linspace(0.01, 10, N)*mm_to_inch;% in mm, x-axis for graph

distance=8%mm
distance=distance*mm_to_inch; %mm between magnet and iron for exact calc




%Magnet setup

%1=cylindrical
diameter_cylindrical=25*mm_to_inch;%mm
length_cylindrical=5*mm_to_inch;%mm

%2=rectangular
A=10*mm_to_inch; %mm width
B=10*mm_to_inch; %mm height
length_rectangular=10*mm_to_inch; %mm length

%3=ring-shaped
outer_diameter=30*mm_to_inch; %mm 
inner_diameter=10*mm_to_inch; %mm 
length_ring=5*mm_to_inch; %mm length
R_ring=outer_diameter/2;
r_ring=inner_diameter/2;

%4=two cylindrical attracting eachother
diameter_cyl1=15*mm_to_inch;%mm
length_cyl1=4*mm_to_inch;%mm

diameter_cyl2=55*mm_to_inch;%mm
length_cyl2=3*mm_to_inch;%mm





%calculations at one point=distance

%1=cylindrical
%flux density, Bx is unit kilotesla
if(steel_back_plate==0)
    Bx_cylindrical=(Br/2)*((length_cylindrical+distance)/sqrt((diameter_cylindrical/2)^2+(length_cylindrical+distance)^2)-(distance/(sqrt((diameter_cylindrical/2)^2+distance^2))))
end
if(steel_back_plate==1)
    Bx_cylindrical=(Br/2)*((2*length_cylindrical+distance)/sqrt((diameter_cylindrical/2)^2+(2*length_cylindrical+distance)^2)-(distance/(sqrt((diameter_cylindrical/2)^2+distance^2))))
end
    %force
Pounds_cylindrical=0.577*(Bx_cylindrical)^2*pi*(diameter_cylindrical/2)^(2)*number_of_magnets;
KG_cylindrical=Pounds_cylindrical*pounds_to_kg
F_cylindrical=KG_cylindrical*g

%2=rectangular
if(steel_back_plate==0)
   Bx_rectangular=(Br/pi)*(atan((A*B)/(2*distance*sqrt(4*distance^2+A^2+B^2)))-atan((A*B)/(2*(length_rectangular+distance)*sqrt(4*(length_rectangular+distance)^2)+A^2+B^2)))
end
if(steel_back_plate==1)
   Bx_rectangular=(Br/pi)*(atan((A*B)/(2*distance*sqrt(4*distance^2+A^2+B^2)))-atan((A*B)/(2*(2*length_rectangular+distance)*sqrt(4*(2*length_rectangular+distance)^2)+A^2+B^2)))
end
%force
Pounds_rectangular=0.577*(Bx_rectangular)^2*A*B*number_of_magnets;
KG_rectangular=Pounds_rectangular*pounds_to_kg
F_rectangular=KG_rectangular*g

%3=ring-shaped
if(steel_back_plate==0)
    Bx_ring=(Br/2)*(((length_ring+distance)/sqrt(R_ring^2+(length_ring+distance)^2))-((length_ring+distance)/sqrt(r_ring^2+(length_ring+distance)^2))-(distance/(sqrt(R_ring^2+distance^2)))+(distance/(sqrt(r_ring^2+distance^2))))
end
if(steel_back_plate==1)
    Bx_ring=(Br/2)*(((2*length_ring+distance)/sqrt(R_ring^2+(2*length_ring+distance)^2))-((2*length_ring+distance)/sqrt(r_ring^2+(2*length_ring+distance)^2))-(distance/(sqrt(R_ring^2+distance^2)))+(distance/(sqrt(r_ring^2+distance^2))))
end

%force
Pounds_ring=0.577*(Bx_ring)^2*((pi*(R_ring)^2)-(pi*(r_ring)^2))*number_of_magnets;
KG_ring=Pounds_ring*pounds_to_kg
F_ring=KG_ring*g






%make graph
if(magnet_type==1) %1=cylindrical
    for(k=1:N)
        %flux density, Bx is unit kilotesla
        if(steel_back_plate==0)
            Bx_cylindrical(k)=(Br/2)*((length_cylindrical+distance_axis(k))/sqrt((diameter_cylindrical/2)^2+(length_cylindrical+distance_axis(k))^2)-(distance_axis(k)/(sqrt((diameter_cylindrical/2)^2+distance_axis(k)^2))));
        end
        if(steel_back_plate==1)
            Bx_cylindrical(k)=(Br/2)*((2*length_cylindrical+distance_axis(k))/sqrt((diameter_cylindrical/2)^2+(2*length_cylindrical+distance_axis(k))^2)-(distance_axis(k)/(sqrt((diameter_cylindrical/2)^2+distance_axis(k)^2))));
        end        
        
            %force
        Pounds_cylindrical(k)=0.577*(Bx_cylindrical(k))^2*pi*(diameter_cylindrical/2)^(2)*number_of_magnets;
        KG_cylindrical(k)=Pounds_cylindrical(k)*pounds_to_kg;
        F_cylindrical(k)=KG_cylindrical(k)*g;  
    end
    %make plot
    figure(1)
    if(force_plot==1)%newton
        plot(distance_axis/mm_to_inch,F_cylindrical,'LineWidth',2)
        ylabel('force [N]')
    end
    if(force_plot==2)%kg
        plot(distance_axis/mm_to_inch,KG_cylindrical,'LineWidth',2)
        ylabel('force [kg]')
    end
    grid on
    title('Cylindrical magnet - Force')
    xlabel('distance [mm]')
    
    figure(2)
    plot(distance_axis/mm_to_inch,Bx_cylindrical,'LineWidth',2)
    ylabel('Flux density [kG]')
    title('Cylindrical magnet - Flux density')
    xlabel('distance [mm]')  
    grid on;
end

if(magnet_type==2) %2=rectangular
    for(k=1:N)
        %flux density, Bx is unit kilotesla
        if(steel_back_plate==0)
            Bx_rectangular(k)=(Br/pi)*(atan((A*B)/(2*distance_axis(k)*sqrt(4*distance_axis(k)^2+A^2+B^2)))-atan((A*B)/(2*(length_rectangular+distance_axis(k))*sqrt(4*(length_rectangular+distance_axis(k))^2)+A^2+B^2)));
        end
        if(steel_back_plate==1)
            Bx_rectangular(k)=(Br/pi)*(atan((A*B)/(2*distance_axis(k)*sqrt(4*distance_axis(k)^2+A^2+B^2)))-atan((A*B)/(2*(2*length_rectangular+distance_axis(k))*sqrt(4*(2*length_rectangular+distance_axis(k))^2)+A^2+B^2)));
        end
        %force
        Pounds_rectangular(k)=0.577*(Bx_rectangular(k))^2*A*B*number_of_magnets;
        KG_rectangular(k)=Pounds_rectangular(k)*pounds_to_kg;
        F_rectangular(k)=KG_rectangular(k)*g;
    end
    %make plot
    figure(1)
    if(force_plot==1)%newton
        plot(distance_axis/mm_to_inch,F_rectangular,'LineWidth',2)
        ylabel('force [N]')
    end
    if(force_plot==2)%kg
        plot(distance_axis/mm_to_inch,KG_rectangular,'LineWidth',2)
        ylabel('force [kg]')
    end
    grid on
    title('Rectangular magnet - Force')
    xlabel('distance [mm]')    
    
    figure(2)
    plot(distance_axis/mm_to_inch,Bx_rectangular,'LineWidth',2)
    ylabel('Flux density [kG]')
    title('Rectangular magnet - Flux density')
    xlabel('distance [mm]')  
    grid on;
end

if(magnet_type==3) %3=Ring
    for(k=1:N)
        if(steel_back_plate==0)
            Bx_ring(k)=(Br/2)*(((length_ring+distance_axis(k))/sqrt(R_ring^2+(length_ring+distance_axis(k))^2))-((length_ring+distance_axis(k))/sqrt(r_ring^2+(length_ring+distance_axis(k))^2))-(distance_axis(k)/(sqrt(R_ring^2+distance_axis(k)^2)))+(distance_axis(k)/(sqrt(r_ring^2+distance_axis(k)^2))));
        end
        if(steel_back_plate==1)
            Bx_ring(k)=(Br/2)*(((2*length_ring+distance_axis(k))/sqrt(R_ring^2+(2*length_ring+distance_axis(k))^2))-((2*length_ring+distance_axis(k))/sqrt(r_ring^2+(2*length_ring+distance_axis(k))^2))-(distance_axis(k)/(sqrt(R_ring^2+distance_axis(k)^2)))+(distance_axis(k)/(sqrt(r_ring^2+distance_axis(k)^2))));
        end

    %force
    Pounds_ring(k)=0.577*(Bx_ring(k))^2*((pi*(R_ring)^2)-(pi*(r_ring)^2))*number_of_magnets;
    KG_ring(k)=Pounds_ring(k)*pounds_to_kg;
    F_ring(k)=KG_ring(k)*g;
    end
    %make plot
    figure(1)
    if(force_plot==1)%newton
        plot(distance_axis/mm_to_inch,F_ring,'LineWidth',2)
        ylabel('force [N]')
    end
    if(force_plot==2)%kg
        plot(distance_axis/mm_to_inch,KG_ring,'LineWidth',2)
        ylabel('force [kg]')
    end
    grid on
    title('Ring magnet - Force')
    xlabel('distance [mm]')
    
    figure(2)
    plot(distance_axis/mm_to_inch,Bx_ring,'LineWidth',2)
    ylabel('Flux density [kG]')
    title('Ring magnet - Flux density')
    xlabel('distance [mm]')  
    grid on;
end

 
if(magnet_type==4) %4=two cylindrical attracting eachother
    for(k=1:N)
        %flux density, Bx is unit kilotesla
        if(steel_back_plate==0)
            Bx_two_cylindrical1(k)=(Br/2)  *  ((length_cyl1+distance_axis(k)/2)/sqrt((diameter_cyl1/2)^2+(length_cyl1+distance_axis(k)/2)^2)  -  ((distance_axis(k)/2)/(sqrt((diameter_cyl1/2)^2+(distance_axis(k)/2)^2))));%   
            Bx_two_cylindrical2(k)=(Br/2)  *  ((length_cyl2+distance_axis(k)/2)/sqrt((diameter_cyl2/2)^2+(length_cyl2+distance_axis(k)/2)^2)  -  ((distance_axis(k)/2)/(sqrt((diameter_cyl2/2)^2+(distance_axis(k)/2)^2))));
            Bx_two_cylindrical(k)=Bx_two_cylindrical1(k)+Bx_two_cylindrical2(k);
        end
        if(steel_back_plate==1)
            Bx_two_cylindrical(k)=(Br/2)  *  ((2*length_cyl1+distance_axis(k)/2)/sqrt((diameter_cyl1/2)^2+(2*length_cyl1+distance_axis(k)/2)^2)  -  ((distance_axis(k)/2)/(sqrt((diameter_cyl1/2)^2+(distance_axis(k)/2)^2)))    +     (2*length_cyl2+distance_axis(k)/2)/sqrt((diameter_cyl2/2)^2+(2*length_cyl2+distance_axis(k)/2)^2)-((distance_axis(k)/2)/(sqrt((diameter_cyl2/2)^2+(distance_axis(k)/2)^2))));
        end        
        
            %force
        Pounds_two_cylindrical(k)=0.577*(Bx_two_cylindrical(k))^2*pi*((diameter_cyl1/2+diameter_cyl2/2)/2)^(2)*number_of_magnets;
        KG_two_cylindrical(k)=Pounds_two_cylindrical(k)*pounds_to_kg;
        F_two_cylindrical(k)=KG_two_cylindrical(k)*g;  %superposition
    end
    %make plot
    figure(1)
    if(force_plot==1)%newton
        plot(distance_axis/mm_to_inch,F_two_cylindrical,'LineWidth',2)
        ylabel('force [N]')
    end
    if(force_plot==2)%kg
        plot(distance_axis/mm_to_inch,KG_two_cylindrical,'LineWidth',2)
        ylabel('force [kg]')
    end
    grid on
    title('Two cylindrical magnets attracting - Force')
    xlabel('distance [mm]')  
    %flux plot
    figure(2)
    plot(distance_axis/mm_to_inch,Bx_two_cylindrical,'LineWidth',2)
    ylabel('Flux density [kG]')
    title('Two cylindrical magnets attracting - Flux density')
    xlabel('distance [mm]')  
    grid on;
end