%robot_forces.m
%Program for calculations of forces on robot hanging on wall with permanent
%magnet connections
%Hvard Braa 

%Effects not taken into account:
    %-difference between static friction and kinetic friction. A kinetic
    % friction number is approximated
    
    %-Sentripental acceleration. Very small at low speed
    
    %- Temperature dependency in friction coeff
    
    %- Friction changes when rubber gets old and hard

clear all
clc



%From practical measurements:
%u_rubber_dry=1.0; %from EPDM
u_rubber_dry=0.95; %from TT-Racket,wheels
u_rubber_wet=0.73;
u_pom_dry=0.11;
u_pom_wet=0.08;
u_wheels_dry=0.67;
u_wheels_wet=0.4;

%mass robot
m_robot=2.5;

%mass cleaner
m_cleaner=0.5;%mass in kg

%magnetic force

%30x15-15x15=1,65kg
%30x15-20x20=2,35kg
%30x15-12x12=0,90kg
g=9.81;
F_magn=124;% magnetic force in Newton, 70N at 10mm measured


%motor force
moment_motors=16.5*2;%kg/cm
diameter_wheels=0.037;%m
Force_motors_max=g*moment_motors/(diameter_wheels/2);

u_robot=0.45;%friction coeff robot-wall (dry rubber)
u_cleaner=0.11;%friction coeff washer-wall (wet rubber)

N=100; %resolution graph
wall_angle=linspace(0.0, pi, N);%radians, 0=vertical. pi/2=horisontal


G_robot=m_robot*g;
G_cleaner=m_cleaner*g;

side_angle=3;%deg, side in tank
bottom_angle=69;%deg, bottom in tank

for(k=1:N)
Gx_robot(k)=G_robot*cos(wall_angle(k));
Gy_robot(k)=G_robot*sin(wall_angle(k));
Gx_cleaner(k)=G_cleaner*cos(wall_angle(k));
Gy_cleaner(k)=G_cleaner*sin(wall_angle(k));
%normal forces
    N_robot(k)=F_magn-Gy_robot(k);% normalforce between robot and wall
    N_cleaner(k)=F_magn+Gy_robot(k);% normalforce between cleaner and wall

    % horisontal direction=moving
    F_friction_robot_horisontal(k)=N_cleaner(k)*u_cleaner; %friction in the cleaner

    %y: direction normal to wall
    F_friction_robot_max(k)=u_robot*N_robot(k); %maximum force before robot slips the wall (wheels)
    F_friction_cleaner(k)=u_cleaner*N_cleaner(k);%friction in the cleaner
                   
    %driving vertical up the tank
    F_wheels_vertical_up(k)=F_friction_cleaner(k) + Gx_cleaner(k) + Gx_robot(k);

    %driving horisontal, sideways force
    F_wheels_horisontal(k)=Gx_cleaner(k)+Gx_robot(k)-F_friction_cleaner(k);
    if(F_wheels_horisontal(k)<0)%F_wheels_horisontal(k) kan ikke bli negativ
        F_wheels_horisontal(k)=0;
    end
    %motor force
    F_motors(k)=(diameter_wheels/2*F_wheels_vertical_up(k))/2;
end

figure(1)%driving vertical up
plot(wall_angle,F_wheels_vertical_up,'b','LineWidth',2)
hold on; 
plot(wall_angle,F_friction_robot_max-F_wheels_vertical_up,'g','LineWidth',2)
plot(wall_angle,F_friction_robot_max,'r','LineWidth',2)
%plot(wall_angle,F_motors,'k','LineWidth',2)
%plot(wall_angle,F_wheels_vertical_up,'cy','LineWidth',2)
%plot(wall_angle,F_motors_max,'k','LineWidth',2)
%plot vertical lines
x=[side_angle*pi/180,bottom_angle*pi/180];
ylim=get(gca,'ylim');
line([x;x],ylim.','linewidth',2,'color',[0,0,0]);
ylabel('Forces [N]')
title('Robot forces, robot driving vertical up')
xlabel('alfa [rad]') 
axis([0 pi/2 -10 60]);
%legend('F friction robot, vertical drive','Force at wheels at total slip','Location','EastOutside')
legend('F_f_r_i_c_t_i_o_n _r_o_b_o_t','F_s_a_f_e_t_y','F_f_r_i_c_t_i_o_n _r_o_b_o_t _m_a_x')
grid on;

%motormoment

figure(3)%driving vertical up
plot(wall_angle,F_motors,'b','LineWidth',2)
%plot(wall_angle,F_wheels_vertical_up,'cy','LineWidth',2)
%plot(wall_angle,F_motors_max,'k','LineWidth',2)
%plot vertical lines
x=[side_angle*pi/180,bottom_angle*pi/180];
ylim=get(gca,'ylim');
line([x;x],ylim.','linewidth',2,'color',[0,0,0]);
ylabel('Motor moment [Nm]')
title('Motor moment, robot driving vertical up')
xlabel('alfa [rad]') 
axis([0 pi/2 0 0.5]);
%legend('F friction robot, vertical drive','Force at wheels at total slip','Location','EastOutside')
legend('M_m_o_t_o_r')
grid on;



figure(2)%driving horisontal
plot(wall_angle,F_wheels_horisontal,'b','LineWidth',2)
hold on; 
plot(wall_angle,F_friction_robot_max-F_wheels_horisontal,'g','LineWidth',2)
plot(wall_angle,F_friction_robot_max,'r','LineWidth',2)

%plot(wall_angle,N_robot,'cy','LineWidth',2)

%plot(wall_angle,F_wheels_vertical_up,'cy','LineWidth',2)
%plot(wall_angle,F_motors_max,'k','LineWidth',2)
%plot vertical lines
x=[side_angle*pi/180,bottom_angle*pi/180];
ylim=get(gca,'ylim');
line([x;x],ylim.','linewidth',2,'color',[0,0,0]);
ylabel('Forces [N]')
title('Robot forces, robot front in horisontal direction')
xlabel('alfa [rad]') 
axis([0 pi/2 -10 90]);
legend('F_f_r_i_c_t_i_o_n _r_o_b_o_t','F_s_a_f_e_t_y','F_f_r_i_c_t_i_o_n _r_o_b_o_t _m_a_x')
grid on;
